/*
 * =====================================================================================
 *
 *       Filename:  fmutils.c
 *
 *    Description:  fmutils c file
 *
 *        Version:  1.0
 *
 *         Author:  Alexander Aring, a.aring@phytec.de
 *        Company:  PHYTEC
 *
 * =====================================================================================
 */

#include "fmutils.h"

static struct kmod_ctx *ctx = NULL;

/* 
 * Load a list of modules.
 * Parameter modules is a char*
 * array which must end with NULL
 */
int fmutils_modprobe(char *modules[])
{
    int err = -1;
    char* modname = NULL;

    struct kmod_module *mod = NULL;

    /* Check if we have already 
     * called modprobe */
    if (ctx) {
        fprintf(stderr, "Error: modprobe already called!\n");
        return -1;
    }

    ctx = kmod_new(NULL, NULL);
    if (!ctx) {
        fprintf(stderr, "Error: kmod_new() failed!\n");
        return -1;
    }

    /* Iterate all module from 
     * modules and load them */
    for (modname = *modules; modname; modname = *(++modules)) {
        err = kmod_module_new_from_name(ctx, modname, &mod);
        if (err <0) {
            fprintf(stderr, "Error: kmod_module_new_from_name() failed!\n");
            goto err_ctx;     
        }                     
                              
        err = kmod_module_insert_module(mod, 0, NULL);
        if (err < 0) {
			if (err == -EEXIST) {
				fprintf(stderr, "Module %s already loaded, reinsert module.\n", modname);
				err = kmod_module_remove_module(mod, 0);
				if (err < 0) {
					fprintf(stderr, "Error: kmod_module_remove_module() failed at module %s!\n", modname);
					goto err_mod;
				}
				err = kmod_module_insert_module(mod, 0, NULL);
				if (err < 0) {
					fprintf(stderr, "Error: kmod_module_insert_module() failed at module %s:%d!\n", modname, err);
					goto err_mod;
				}
				kmod_module_unref(mod);
				continue;
			}
            fprintf(stderr, "Error: kmod_module_insert_module() failed at module %s!\n", modname);
            goto err_mod;    
        }                    
        
        kmod_module_unref(mod);
    } 
      
    return 0;
err_mod:
    kmod_module_unref(mod);
err_ctx:
    kmod_unref(ctx);
	return -1;
}     

/* 
 * Unload a list of modules.
 * Parameter modules is a char*
 * array which must end with NULL
 */
int fmutils_rmmod(char *modules[])
{
    int err = -1;
    char* modname = NULL;

    struct kmod_module *mod = NULL;

    if (!ctx) {
        fprintf(stderr, "Call modprobe first!\n");
        return -1;
    }  
       
    /* Iterate all module from 
     * modules and unload them */
    for (modname = *modules; modname; modname = *(++modules)) {
        err = kmod_module_new_from_name(ctx, modname, &mod);
        if (err <0) {
            fprintf(stderr, "Error: kmod_module_new_from_name() failed!\n");
            goto err_ctx;
        }

        err = kmod_module_remove_module(mod, 0);
        if (err < 0) {
            fprintf(stderr, "Error: kmod_module_remove_module() failed at module %s!\n", modname);
            goto err_mod;
        }

        kmod_module_unref(mod);
    }

    kmod_unref(ctx);

	return 0;
err_mod:
    kmod_module_unref(mod);
err_ctx:
    kmod_unref(ctx);
	return -1;
}
