/*
 * Copyright (c) 2014 PHYTEC Messtechnik GmbH
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 */

#include <stdio.h>
#include <opencv/cv.h>
#include <opencv2/highgui/highgui.hpp>
#include <opencv2/opencv.hpp>
#include <string.h>
#include <unistd.h>

using namespace cv;

int main(int argc, char* argv[]) {
char *camtype = NULL;
	int index;
	int c;
	double frame_width;
	double frame_height;
	double x_div;
	double y_div;

	opterr = 0;
	while ((c = getopt (argc, argv, "c:x:y:")) != -1)
		switch (c)
		{
		case 'c':
			camtype = optarg;
			if (strcmp(camtype,"VM-006")==0){
				frame_width=1280;
				frame_height=1024;
				break;
			}else if ((strcmp(camtype,"VM-009")==0)){
				frame_width=1280;
				frame_height=1024;
				break;
			}else if ((strcmp(camtype,"VM-010")==0)){
				frame_width=752;
				frame_height=480;
				break;
			}else if ((strcmp(camtype,"VM-011")==0)){
				sleep(1);
                                fprintf (stderr, "\nVM-011 is not supported at the moment.\n");
                                fprintf (stderr, "It is not possible to grab a frame in full resolution.\n");
                                fprintf (stderr, "Also the color format is not correct!\n\n");
				sleep(1);
				frame_width=2592;
				frame_height=1944;
				break;
			}else if ((strcmp(camtype,"VM-008")==0)){
				fprintf (stderr, "VM-008 is not supported at the moment.\n");
				return 1;
			}else{
				fprintf (stderr, "-c has an incorret argument.\n");
                                return 1;
			}
		case '?':
			if (optopt == 'c'){
				fprintf (stderr, "-%c requires an argument.\n", optopt);
				return 1;
			}
			else if (isprint (optopt)){
				fprintf (stderr, "Unknown option `-%c'.\n", optopt);
				return 1;
			}
			else{
				fprintf (stderr, "Unknown option character `\\x%x'.\n", optopt);
				return 1;
			}
		default:
			abort ();
	}

	for (index = optind; index < argc; index++)
		printf ("Non-option argument %s\n", argv[index]);

	if (camtype == NULL){
		fprintf (stderr, "The Name of the Camera Module is missing.\n");
                fprintf (stderr, "Use -c <Name of the Camera Module>\n");
                fprintf (stderr, "Name of the Camera Module: VM-006, VM-009, VM-010 or VM-011.\n");
                fprintf (stderr, "With the parameter -s you can scale the resolution to 640x480 on the Display.\n");
		fprintf (stderr, "For example: -c VM-009 -s\n");
		return 1;
	}

	Mat frame;
	Mat resized_frame;
	CvCapture * capture = NULL; // capture for camera
	// take the first camera we can get
	capture = cvCaptureFromCAM( CV_CAP_ANY); // find other options in opencv2/highgui/highui_c.h
	// check if we got a camera
	if( capture == NULL) {
		printf( "Error: no camera found\r\n");
		return -1;
	}
	// looks like we got a camera - let's set the desired resolution
	if ((strcmp(camtype,"VM-011")!=0)){
		cvSetCaptureProperty( capture, CV_CAP_PROP_FRAME_WIDTH,  frame_width);
		cvSetCaptureProperty( capture, CV_CAP_PROP_FRAME_HEIGHT, frame_height);
	}
	// we're all set up - let's request an image

	//skip frames
	int i = 0;
	while (i < 10){
		frame = cvQueryFrame( capture);
		i++;
	}

	if( frame.empty()) {
		printf( "Couldn't retrieve a frame from camera\r\n");
		return -1;
	}
	// check image dimensions
	if( frame.cols < frame_width || frame.rows < frame_height) {
		printf( "Image dimensions didn't match\r\n");
		return -1;
	}

        // store image
        imwrite( "/home/opencv_examples/opencv_testimage.jpg", frame);

        // use the following line to display the image on screen

	namedWindow("Image");
	if (frame_width > 640){
	        resizeWindow("Image", 640, 480);
		x_div = 640/frame_width;
		y_div = 480/frame_height;
		resize(frame,resized_frame,Size(),x_div,y_div,INTER_LINEAR);
		imshow( "Image", resized_frame);
	}
	else{
		imshow( "Image", frame);
	}

        // wait for keyboard input ('q') to quit application
        printf( "Hit 'STRG -c' to quit application\r\n");
        while( 'q' != waitKey( 30));

	// finally release the capture
	cvReleaseCapture( &capture);
	return 0;
}
